if (!VeohCompass) {
    VeohCompass = {};
}

VeohCompass.bookmark = (function () {
    /** Extension Id, matches id specified in install.rdf */
    var EXT_ID = 'searchrecs@veoh.com',

    /**
     * Load a named script from the filesystem. Assumes that script resides in
     * <extension>/content.
     *
     * @param string name
     *      The file name of the script to load.
     */
    getScript = function (name) {
        //
        // Use the extension manager to determine the location of js files
        // Install.rdf is assumed to be at the root path for the extension
        //
        var extMgr = Components.classes['@mozilla.org/extensions/manager;1'].getService(Components.interfaces.nsIExtensionManager),
        file = Components.classes['@mozilla.org/file/local;1'].createInstance(Components.interfaces.nsILocalFile),
        cstream = Components.classes['@mozilla.org/intl/converter-input-stream;1'].createInstance(Components.interfaces.nsIConverterInputStream),
        fstream = Components.classes['@mozilla.org/network/file-input-stream;1'].createInstance(Components.interfaces.nsIFileInputStream),
        extDir = extMgr.getInstallLocation(EXT_ID).getItemFile(EXT_ID, 'install.rdf'),

        // JS files are assumed to reside in <extension>/chrome/content
        file.initWithFile(extDir.parent);
        file.append('content');
        file.append(name);

        //
        // Initialize streams and read all data.  Must be sure to allocate a big
        // enough buffer for file contents.  I used fileSize + 2 (just in case).
        //
        if (file.exists()) {
            fstream.init(file, -1, 0, 0);
            cstream.init(fstream, 'UTF-8', file.fileSize + 2, 0);
            cstream.readString(-1, data);
            cstream.close();
        }

        return data.value;
    },

    /**
     * Load support javascript files and initialize bookmarking in a sandboxed
     * environment.  Called when page is fully loaded.  While a sandbox is not
     * strictly necessary, it makes it easier to reuse the javascript in IE.
     *
     * @param Event e
     *      Event object for event that triggered this callback.
     */
    pageLoaded = function (e) {
        var safeWindow = null,
        sandbox = null,
        script = '';

        //
        // Check that this load event was triggered by the HTMLDocument.
        // Firefox documentation specifies that other objects, like
        // images, may also fire this event.
        //
        if (e.originalTarget instanceof HTMLDocument) {
            //
            // Check that window is a top level window: We do not want to inject
            // our script into IFrames.
            //
            safeWindow = e.originalTarget.defaultView;

            if (safeWindow.parent === safeWindow) {
                //
                // Initialize sandbox environment, make sure to pass window and
                // document references along to sandbox.
                //
                sandbox = Components.utils.Sandbox(safeWindow);

                sandbox.window = safeWindow;
                sandbox.document = safeWindow.document;
                sandbox.__proto__ = safeWindow;

                script += getScript('veoh.elements.js');
                script += getScript('veoh.ingest.js');
                script += getScript('veoh.init.js');

                Components.utils.evalInSandbox(script, sandbox);
            }
        }
    };

    return {
        /**
         * Attach event listener to the document to call pageLoaded callback
         * once the document is fully loaded.
         */
        init: function () {
            document.getElementById('appcontent').addEventListener('DOMContentLoaded', pageLoaded, false);
        }
    };
}());

window.addEventListener('load', function () { VeohCompass.bookmark.init(); }, false);

